import { FetchError } from 'ofetch';
import type { FetchResponse } from 'ofetch';
import type { Config, RequestMethod, StatusHandler } from '../types/core';
/**
 * Ensure that the response is a Precognition response.
 */
export declare function validatePrecognitionResponse(response: FetchResponse<unknown>): void;
/**
 * Resolve the handler for the given HTTP response status.
 */
export declare function resolveStatusHandler(config: Config, code: number): StatusHandler | undefined;
/**
 * Determine if the status is successful.
 */
export declare function isSuccess(status: number): boolean;
/**
 * Determine if the error was not triggered by a server response.
 */
export declare function assertIsServerGeneratedError(error: unknown): asserts error is FetchError & {
    response: FetchResponse<unknown>;
};
/**
 * Returns the headers value if present, case insensitive
 * @param headers
 * @param name
 */
export declare function getHeaders(headers: HeadersInit, name: string): string | null;
/**
 * Update the provided headers with name and value provided, case insensitive
 * @param headers
 * @param name
 * @param value
 */
export declare function setHeaders(headers: HeadersInit, name: string, value: string): void;
/**
 * Resolve the configuration.
 */
export declare function resolveConfig(config?: Config): Config;
/**
 * Determine if the payload has any files.
 */
export declare function hasFiles(data: unknown): boolean;
/**
 * Determine if the value is a file.
 */
export declare function isFile(value: unknown): value is Blob | File | FileList;
/**
 * Merge the client specified arguments with the provided configuration.
 */
export declare function mergeConfig(method: RequestMethod, data?: Record<string, unknown>, config?: Config): Config;
export declare function resolveString<T extends string>(value: T | (() => T)): T;
export declare function resolveInitialData<T extends Record<string, unknown>>(data: T | (() => T)): T;
export declare function assertIsFetchResponse(value: unknown, message?: string): asserts value is FetchResponse<unknown>;
export declare function isFetchResponse(value: unknown): value is FetchResponse<unknown>;
